import cv2
import numpy as np
from matplotlib import pyplot as plt
import sys
img = cv2.imread(sys.argv[1])
height, width, channels = img.shape


def sort_points(rect):
	sort_x = rect[np.argsort(rect[:, 0]), :]
	left_points = sort_x[:2, :]
	right_points = sort_x[2:, :]

	left = left_points[np.argsort(left_points[:, 1]), :]

	(top_left, bot_left) = left

	dist1 = np.linalg.norm(np.array(left[0]) - np.array(right_points[0]))
	dist2 = np.linalg.norm(np.array(left[0]) - np.array(right_points[1]))

	if (dist1 > dist2):
		(top_right, bot_right) = (right_points[1], right_points[0])
	else:
		(top_right, bot_right) = (right_points[0], right_points[1])

	return np.array([top_left, top_right, bot_left, bot_right], np.float32)



# threshold in HSV
upper_red1 = np.array([10, 255, 255])
upper_red2 = np.array([179, 255, 255])

lower_red1 = np.array([0, 100, 50])
lower_red2 = np.array([160,100, 50])

hsv_img = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)

# hsv_img = cv2.GaussianBlur(hsv_img, (3,3), 5)
r2 = cv2.inRange(hsv_img, lower_red2, upper_red2)
r1 = cv2.inRange(hsv_img, lower_red1, upper_red1)

out_mask = cv2.addWeighted(r1, 1.0, r2, 1.0, 0)

anded = cv2.bitwise_and(img, img, mask=out_mask)
# cv2.waitKey(0)
#cv2.imshow('maksed', out_mask)
blur = cv2.GaussianBlur(anded, (3,3), 2)

#cv2.imshow('mask blur', blur)
#cv2.waitKey(0)

# blur = anded
th = cv2.adaptiveThreshold(cv2.cvtColor(blur, cv2.COLOR_BGR2GRAY),255,cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY,9,2)
#cv2.imshow('taost', th)
th = ~th
kernel = np.ones((3,3), 'uint8')
th = cv2.dilate(th, kernel, iterations=2)
th = cv2.erode(th, kernel, iterations= 2)
contours = cv2.findContours(th.copy(), cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
contours = contours[0]

# all_points = np.concatenate(contours)
all_points = np.empty([1,1,2])
#cimg = np.zeros_like(img)
#cv2.drawContours(cimg, contours, -1, 255, -1)
#cv2.imshow('contour', cimg)
#cv2.waitKey(0)
# all_points = []
for c in contours:
	perimeter = cv2.arcLength(c, True)
	if perimeter > 200:
		# bounding box method 1
		# interior = np.where(cimg==255)
		# test = np.array(np.transpose([[interior[1]], [interior[0]]]))
		# all_points = np.append(all_points, test, axis=0)

		# # bounding box method 2
		# mom = cv2.moments(c)
		# cx = int(mom['m10']/mom['m00'])
		# cy = int(mom['m01']/mom['m00'])
		# all_points = np.append(all_points, np.array([[[cx, cy]]]), axis=0)

		# bounding box method 3
		all_points = np.append(all_points, c, axis=0)










# print all_points
all_points = np.delete(all_points, 0, 0)
np_points = np.array(all_points, dtype=int)

upper_left = all_points[all_points[:,:,0] < width/2]
upper_left = upper_left[upper_left[:,1] < height/2]

lower_left = all_points[all_points[:,:,0] < width/2]
lower_left = lower_left[lower_left[:,1] > height/2]

upper_right = all_points[all_points[:,:,0] > width/2]
upper_right = upper_right[upper_right[:,1] < height/2]

lower_right = all_points[all_points[:,:,0] > width/2]
lower_right = lower_right[lower_right[:,1] > height/2]

twiddle_x = 0
twiddle_y = 0

max_pt = [width/2, height/2]
max_val = 0
for pt in upper_left:
	dist = np.linalg.norm(pt - [width/2, height/2])
	if dist > max_val:
		max_pt = pt
		max_val = dist
ul = max_pt + [twiddle_x, twiddle_y]
print ul


max_pt = [width/2, height/2]
max_val = 0
for pt in lower_left:
	dist = np.linalg.norm(pt - [width/2, height/2])
	if dist > max_val:
		max_pt = pt
		max_val = dist
ll = max_pt + [twiddle_x, -twiddle_y]
print ll

max_pt = [width/2, height/2]
max_val = 0
for pt in upper_right:
	dist = np.linalg.norm(pt - [width/2, height/2])
	if dist > max_val:
		max_pt = pt
		max_val = dist
ur = max_pt + [-twiddle_x, twiddle_y]
print ur

max_pt = [width/2, height/2]
max_val = 0
for pt in lower_right:
	dist = np.linalg.norm(pt - [width/2, height/2])
	if dist > max_val:
		max_pt = pt
		max_val = dist
lr = max_pt + [-twiddle_x, -twiddle_y]
print lr



# leftmost = all_points[all_points[:,:,0].argmin()]
# rightmost = all_points[all_points[:,:,0].argmax()]
# topmost = all_points[all_points[:,:,1].argmin()]
# botmost = all_points[all_points[:,:,1].argmax()]

# print leftmost, rightmost, topmost, botmost

# rect = minAreaRect(np_points)
box_sorted = np.array([ul, ur, ll, lr], dtype=np.float32)
# print box_sorted
# box = np.int0([leftmost, rightmost, topmost, botmost])
# box = cv2.cv.BoxPoints(rect)
# box = np.int0(box)
# cv2.drawContours(img, [box], -1, (0, 255, 0), 2)
# cv2.imshow('test2', img)
# cv2.waitKey(0)
# box_sorted = sort_points(box)

destination_pts = np.array([ [0,0], [607,0], [0, 683], [607, 683]], np.float32)
# print box_sorted, destination_pts
M = cv2.getPerspectiveTransform(box_sorted, destination_pts)
post_transform = cv2.warpPerspective(img, M, (608, 684))

if (len(sys.argv) > 2):
	img2 = cv2.imread(sys.argv[2])

	cv2.GaussianBlur(img2, (3,3), 2)
	post_transform = cv2.warpPerspective(img2, M, (608, 684))


depth = cv2.imread(sys.argv[3])
post_transform_depth = cv2.warpPerspective(depth, M, (608, 684))
cv2.imwrite('depth_transform.bmp', post_transform_depth)
cv2.imwrite('output.bmp', post_transform)
cv2.circle(img, tuple(np.int0(ll)), 3, thickness=-1, color=(0, 255, 0))

cv2.circle(img, tuple(np.int0(lr)), 3, thickness=-1, color=(0, 255, 0))
cv2.circle(img, tuple(np.int0(ul)), 3, thickness=-1, color=(0, 255, 0))


cv2.circle(img, tuple(np.int0(ur)), 3, thickness=-1, color=(0, 255, 0))
cv2.imwrite('points_selected.bmp', img)

#cv2.imshow('test', post_transform)
#cv2.waitKey(0)
# cropped_img = 
