/**************************************************************************************************************************************************
*                                 Copyright ? 2012 Texas Instruments Incorporated - http://www.ti.com/                                            *
***************************************************************************************************************************************************
*  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: *
*                                                                                                                                                 *
*    Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.                 *
*                                                                                                                                                 *
*    Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the        *
*    documentation and/or other materials provided with the distribution.                                                                         *
*                                                                                                                                                 *
*    Neither the name of Texas Instruments Incorporated nor the names of its contributors may be used to endorse or promote products derived      *
*    from this software without specific prior written permission.                                                                                *
*                                                                                                                                                 *
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT          *
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     *
*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT         *
*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY    *
*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE      *
*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                                                           *
***************************************************************************************************************************************************/
/**
*
* @file    tcp_client.c
*
* @brief	TCP implementation using <system supported function > APIs.
**/
/**************************************************************************************************************************************************/

#include "tcp_client.h"

#ifdef _WINDOWS
#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <winsock2.h>
#include <ws2tcpip.h>
#include <stdlib.h>
#include <stdio.h>

// Need to link with Ws2_32.lib, Mswsock.lib, and Advapi32.lib
#pragma comment (lib, "Ws2_32.lib")
#pragma comment (lib, "Mswsock.lib")
#pragma comment (lib, "AdvApi32.lib")

int TCP_Connect(char *host, unsigned long int port)
{
    WSADATA wsaData;
    struct sockaddr_in   server;
    struct hostent *hptr;
    int sock;
    int ret;
    FILE *file;
    
    WSAStartup(MAKEWORD(2,2), &wsaData);

    hptr = gethostbyname(host);

    if(!hptr)
    {
        fprintf(stderr, "Invalid Host Name\n");
        WSACleanup();
        return -1;
    }

    sock = socket(AF_INET, SOCK_STREAM, 0);
    memcpy((char *)&server.sin_addr,  hptr->h_addr_list[0], hptr->h_length);
    server.sin_family = AF_INET;
    server.sin_port = htons((short)port);

    ret = connect(sock, (struct sockaddr *)&server, sizeof(server));
    
    if(ret < 0)
    {
        fprintf(stderr, "Unable to connect %d\n", ret);
        WSACleanup();
        return -1;
    }

    return sock;	
}

int TCP_Send(int sock, unsigned char *buffer, int length)
{
	/*int k;
	
	for (k = 0; k < length; k++) {
		printf("%d ", (int) buffer[k]);
	}
	printf("\n");*/
	
    // Send 'length' number of bytes from buffer via provided
    // socket <sock> address
	int sent = 0;
	while(sent < length)
	{
		int ret =  send(sock, buffer + sent, length - sent, 0);

		if(ret <= 0)
			return -1;

		sent += ret;
	}

    return 0;
}

int TCP_Receive(int sock, unsigned char *buffer, int length)
{
    //Retrieve 'length' number of bytes into 'buffer' from the socket <sock> address
	int rcvd = 0;

	while(rcvd < length)
	{
		int ret =  recv(sock, buffer + rcvd, length - rcvd, 0);

		if(ret <= 0)
			return -1;

		rcvd += ret;
	}

    return 0;
}

int TCP_Disconnect(int sock)
{    
    //free and cleanup socket occupied memory
    shutdown(sock, SD_SEND);
    closesocket(sock);
    WSACleanup();

    return 0;
}

#endif


#if defined(_LINUX) || defined(_QNX) || defined(_OSX)

/*Include header files related to TCP/IP communication */
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

int TCP_Connect(char *host, unsigned long int port)
{
/*
    int socket = 0;
    //1. Declare and Initialize TCP/IP module
    //2. Create a socket for connecting to server
    //3. Connect to server.
    return socket;
*/
    struct sockaddr_in   server;
    struct hostent *hptr;
    int sock;
    int ret;

    hptr = gethostbyname(host);

    if(!hptr)
    {
        fprintf(stderr, "Invalid Host Name\n");
        return -1;
    }

    sock = socket(AF_INET, SOCK_STREAM, 0);
    memcpy((char *)&server.sin_addr,  hptr->h_addr_list[0], hptr->h_length);
    server.sin_family = AF_INET;
    server.sin_port = htons((short)port);

    ret = connect(sock, (struct sockaddr *)&server, sizeof(server));
    if(ret < 0)
    {
        fprintf(stderr, "Unable to connect %d\n", ret);
        return -1;
    }

    return sock;	
}

int TCP_Send(int sock, unsigned char *buffer, int length)
{
	/*int k;
	
	for (k = 0; k < length; k++) {
		printf("%d ", (int) buffer[k]);
	}
	printf("\n");*/
	
    // Send 'length' number of bytes from buffer via provided
    // socket <sock> address
	int sent = 0;
	while(sent < length)
	{
		int ret =  send(sock, buffer + sent, length - sent, 0);

		if(ret <= 0)
			return -1;

		sent += ret;
	}

    return 0;
}

int TCP_Receive(int sock, unsigned char *buffer, int length)
{
    //Retrieve 'length' number of bytes into 'buffer' from the socket <sock> address
	int rcvd = 0;

	while(rcvd < length)
	{
		int ret =  recv(sock, buffer + rcvd, length - rcvd, 0);

		if(ret <= 0)
			return -1;

		rcvd += ret;
	}

    return 0;
}

int TCP_Disconnect(int sock)
{
    //free and cleanup socket occupied memory
    close(sock);

    return 0;
}

#endif
